DEFAULT_IMAGE_HEIGHT = 240
DEFAULT_IMAGE_WIDTH = 320
DEFAULT_CAMERA_SOURCE = 0
MIN_OPENCV_CAMERA_PROPERTY = 0
MAX_OPENCV_CAMERA_PROPERTY = 18
DEFAULT_KERNEL_SIZE = (3, 3)
OMIT_DIMENSION_VALUES = (-1, 0)
DEFAULT_FAILED_DETECTION_VALUE = "Failed to detect targets"
VISION_LOGGER = "vision"
MULTI_VISION_DELAY = 0.1
BASE_LOGGER = "ovl"
MULTIVISION_LOGGER = "multivision"
