class InvalidCustomFilterError(Exception):
    pass


class MissingResourceError(Exception):
    pass


class VersionError(Exception):
    pass


class CameraError(Exception):
    pass


class ImageError(Exception):
    pass


class FailedDetection(Exception):
    pass


class NotEnoughTargets(Exception):
    pass
